{
    CAN232 Monitor Application written for Delphi 5
    by:
      John Dammeyer
      Automation Artisans Inc.
      www.autoartisans.com
      25OCT02
    for:
      Lawicel Sweden
      www.candip.com

    uses:
        * ComPort Library ver. 2.63                        *
        *   for Delphi 3, 4, 5, 6 and C++ Builder 3, 4, 5  *
        * written by Dejan Crnila, 1998 - 2001             *
        * email: dejancrn@yahoo.com                        *

}

unit CAN232Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Menus, Dialogs,
  StdCtrls, ExtCtrls, Registry, CPort, CPortCtl, CAN232Link, Xtended;
const
    VERSION = 105;

type

  CANREC = record
    ID : Int64;
    DLC : Integer;
    Data : array[1..8] of Integer;
  end;

  TCAN232Form = class(TForm)
    ComPort: TComPort;
    Memo: TMemo;
    Button_Open: TButton;
    CANID_Field: TEdit;
    Button_Send: TButton;
    ID11Bit_CB: TCheckBox;
    Label1: TLabel;
    CANButton_Settings: TButton;
    DLC_Field: TEdit;
    Label2: TLabel;
    CANData0_Field: TEdit;
    CANData1_Field: TEdit;
    CANData2_Field: TEdit;
    CANData3_Field: TEdit;
    CANData4_Field: TEdit;
    CANData5_Field: TEdit;
    CANData6_Field: TEdit;
    CANData7_Field: TEdit;
    Timer1: TTimer;
    VersionButton: TButton;
    FlagsButton: TButton;
    MainMenu1: TMainMenu;
    About1: TMenuItem;
    File1: TMenuItem;
    OpenLogFile1: TMenuItem;
    SaveAsLogFile: TMenuItem;
    SaveParameters1: TMenuItem;
    LoadParameters1: TMenuItem;
    Exit1: TMenuItem;
    NewLogFile1: TMenuItem;
    SaveLogFileDialog: TSaveDialog;
    OpenLogFileDialog: TOpenDialog;
    SaveLogFile: TMenuItem;
    procedure Button_OpenClick(Sender: TObject);
    procedure COMButton_SettingsClick(Sender: TObject);
    procedure Button_SendClick(Sender: TObject);
    procedure ComPortOpen(Sender: TObject);
    procedure ComPortClose(Sender: TObject);
    procedure ComPortRxChar(Sender: TObject; Count: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CANButton_SettingsClick(Sender: TObject);
    procedure VersionButtonClick(Sender: TObject);
    procedure FlagsButtonClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure NewLogFile1Click(Sender: TObject);
    procedure OpenLogFile1Click(Sender: TObject);
    procedure SaveAsLogFileClick(Sender: TObject);
    procedure SaveParameters1Click(Sender: TObject);
    procedure LoadParameters1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Msg : CANREC;                // Holds message to be sent.
    CAN232Device : TCAN232_REC;  // Describes CAN232 parameters.
    CAN232OutputBuffer : string; // Characters received from COM port
end;

var
  CAN232Form: TCAN232Form;

implementation

uses CAN232Set, AboutBox;
{$R *.DFM}

var
  CAN232SetupForm: TCAN232SetupForm;

{
  Called when main form is created.
    Sets up version number onto ICON when minimized on task bar.
    Sets default CAN rate.
    If the CAN232 was already connected, it's closed and then reopened.
}
procedure TCAN232Form.FormCreate(Sender: TObject);
var versiontemp : integer;
    versionstr : string;
begin
    versiontemp := VERSION mod 100;
    if versiontemp < 10 then
        versionstr := '0' + IntToStr(versiontemp)
    else
        versionstr := IntToStr(VersionTemp);

    Application.Title := 'CAN232 Monitor Rev ' + IntToStr(VERSION DIV 100) + '.' + versionstr;

    CAN232Device.CAN_RATE := 1;
    CAN232Device.CAN232_RATE := 2;
    if ComPort.Connected then begin
        ComPort.WriteStr(#13);      // Clear out CAN232 Buffer.
        ComPort.WriteStr('C'+#13);      // Close CAN232
        ComPort.WriteStr('V'+#13);  // Request Version.
        ComPort.WriteStr('S' + IntToStr(CAN232Device.CAN_RATE) + #13);
        ComPort.WriteStr('O'+#13);
        Memo.Lines.Add('O');
    end;
end;


{
  Used to convert a string of hex characters into an integer.
    Works better than Delphi library routine.
}
function HexToInt( buf : string ) : int64;
var
    i : integer;
    LargeResult : integer;
begin
    LargeResult := 0;
    buf := uppercase(buf);
    for i := 1 to length(buf) do begin
        if buf[i] in ['0'..'9'] then
            LargeResult := (LargeResult shl 4) + ord(buf[i]) - $30
        else if buf[i] in ['A'..'F'] then
            LargeResult := (LargeResult shl 4) + (ord(buf[i]) - $41 + 10);
    end;
    HexToInt := LargeResult;
end;

{
  Evoked when Open/Close Button is clicked.
      Sets Poll rate for Timer that sends 'A' command to CAN232.
      If CAN port was open it closes it.
      If CAN port closed then it opens it.
}
procedure TCAN232Form.Button_OpenClick(Sender: TObject);
begin
  if CAN232Device.CAN_POLL_RATE >= 50 then
      Timer1.Interval := CAN232Device.CAN_POLL_RATE;

  if ComPort.Connected then begin
    ComPort.WriteStr('C'+#13);
    Memo.Lines.Add('C');
    ComPort.Close;
  end
  else begin
    ComPort.Open;
    ComPort.WriteStr(#13);      // Clear out CAN232 Buffer.
    ComPort.WriteStr('C'+#13);      // Close CAN232
    ComPort.WriteStr('V'+#13);  // Request Version.
    ComPort.WriteStr('S' + IntToStr(CAN232Device.CAN_RATE) + #13);
    Memo.Lines.Add('O');
    ComPort.WriteStr('O'+#13);
  end;
end;

{
  Set CAN and RS232 COM port parameters.
}
procedure TCAN232Form.COMButton_SettingsClick(Sender: TObject);
begin
  ComPort.ShowSetupDialog;
end;

{
  Send message entered into the fields at bottom of form if the
  CAN232 is online.  If not display a message.
}
procedure TCAN232Form.Button_SendClick(Sender: TObject);
var
  Str: String;
  i : integer;
begin
  Msg.ID := HexToInt(CANID_Field.Text);
  Msg.DLC := HexToInt(DLC_Field.Text);
  Msg.Data[1] := HexToInt(CANData0_Field.Text);
  Msg.Data[2] := HexToInt(CANData1_Field.Text);
  Msg.Data[3] := HexToInt(CANData2_Field.Text);
  Msg.Data[4] := HexToInt(CANData3_Field.Text);
  Msg.Data[5] := HexToInt(CANData4_Field.Text);
  Msg.Data[6] := HexToInt(CANData5_Field.Text);
  Msg.Data[7] := HexToInt(CANData6_Field.Text);
  Msg.Data[8] := HexToInt(CANData7_Field.Text);

  if ID11Bit_CB.Checked then begin
    Str := 't' + IntToHex(Msg.ID and $7ff,3);
  end
  else begin
    Str := 'T' + IntToHex(Msg.ID and $1fffffff,8);
  end;
  Str := Str + IntToHex(Msg.DLC and $f,1);
  for i := 1 to Msg.DLC do begin
        Str := Str + IntToHex(Msg.Data[i] and $FF,2);
  end;

    if ComPort.Connected then begin
        Memo.Lines.Add(Str);
        Str := Str + #13;
        ComPort.WriteStr(Str)
    end
        else MessageDlg('Comport Not Open', mtInformation, [mbOk], 0);

end;

{
  Called by COMPort after COM port has been opened.  Changes text on Button.
}
procedure TCAN232Form.ComPortOpen(Sender: TObject);
begin
  Button_Open.Caption := 'Close COM Port';
  CAN232OutputBuffer := '';

end;

{
  Called COMPort after COM port has been closed.  Changes text on Button.
}
procedure TCAN232Form.ComPortClose(Sender: TObject);
begin
  if Button_Open <> nil then
    Button_Open.Caption := 'Open COM Port';
end;

{
  Called from COMPort after a number of characters have arrived.
    The characters are parsed for the CR character and when it's there
    the input is treated as a complete line from the CAN232.
    If the 'A' response is received it's ignored since an 'A' could
    be returned every 50mS which scrolls valid data off the screen.
    In this application we don't care if we don't get an 'A' back.
    When the number of lines scrolled to the screen surpasses 22 we delete
    the first one so the list doesn't grow too large and become unmanagable.
}
procedure TCAN232Form.ComPortRxChar(Sender: TObject; Count: Integer);
var
  InputBuffer: String;
  i : integer;
begin
  ComPort.ReadStr(InputBuffer, Count);
  for i := 1 to Count do begin
      if InputBuffer[i] <> #13 then
          CAN232OutputBuffer := CAN232OutputBuffer + InputBuffer[i]
      else  begin
          if (Length(CAN232OutputBuffer) > 1) and
             (CAN232OutputBuffer[1] <> 'A') then begin
                Memo.Lines.Add(CAN232OutputBuffer);
                ExtendedLogData(CAN232OutputBuffer);
          end;
          CAN232OutputBuffer := '';
      end;
  end;
  if Memo.Lines.Count > 22 then  Memo.lines.Delete(0);
end;

{
  Timer Thread function which is activiated each time timer expires.
  This is used to poll the CAN232.
}
procedure TCAN232Form.Timer1Timer(Sender: TObject);
begin
    if ComPort.Connected then ComPort.WriteStr('A'+#13);
end;

{
  Launchs the dialog for setting CAN232 parameters like RS232 baud rate
  and CAN bit rate.  Timer interval for polling CAN232 is also set here.
  Once CAN parameters are set,  then the COM port parameters are set.
  If user clicks OK then new parameters are copied into data structure.
}
procedure TCAN232Form.CANButton_SettingsClick(Sender: TObject);
var
    CANInfo : TCAN232_REC;
    ptrCANInfo : PCAN232_REC;
begin
    // Pass parameters.
    if CAN232Device.Initialized then begin
        CANInfo.CAN_RATE := CAN232Device.CAN_RATE;
        CANInfo.CAN232_RATE := CAN232Device.CAN232_RATE;
        CANInfo.CAN_POLL_RATE := CAN232Device.CAN_POLL_RATE
    end
    else begin
        CANInfo.CAN_RATE := 1;
        CANInfo.CAN232_RATE := 2;
        CANInfo.CAN_POLL_RATE := 100;
    end;

    ptrCANInfo := @CANInfo;
    // Create the dialog
    CAN232SetupForm := TCAN232SetupForm.CreateFrameData(ptrCANInfo, Self);
    // Shw the dialog.
    if CAN232SetupForm.ShowModal = mrOK then begin
        // If OK was pressed update the changed parameters.
        CAN232Device.CAN_RATE := CANInfo.CAN_RATE;
        CAN232Device.CAN232_RATE := CANInfo.CAN232_RATE;
        // but range check on number of milliseconds between
        CAN232Device.CAN_POLL_RATE := CANInfo.CAN_POLL_RATE;
        CAN232Device.Initialized := TRUE;
        ComPort.BaudRate := StrToBaudRate(CAN232BaudRates[CAN232Device.CAN232_RATE]);
        ComPort.ShowSetupDialog;
        if ComPort.Connected then begin
            ComPort.WriteStr(#13);      // Clear out CAN232 Buffer.
            ComPort.WriteStr('C'+#13);      // Close CAN232
            ComPort.WriteStr('V'+#13);  // Request Version.
            ComPort.WriteStr('S' + IntToStr(CANInfo.CAN_RATE) + #13);
            Memo.Lines.Add('S' + IntToStr(CANInfo.CAN_RATE));
            ComPort.WriteStr('U' + IntToStr(CANInfo.CAN232_RATE) + #13);
            Memo.Lines.Add('U' + IntToStr(CANInfo.CAN232_RATE));
            ComPort.WriteStr('O'+#13);
        end;

    end;
    // Dispose of the parameters.
    CAN232SetupForm.Free;
end;

{
  Ask the CAN232 to report its Version #
}
procedure TCAN232Form.VersionButtonClick(Sender: TObject);
begin
    if ComPort.Connected then begin
        ComPort.WriteStr(#13+'V'+#13);  // Request Version.
        Memo.Lines.Add('V');
    end;
end;

{
  Ask the CAN232 to report the flag values.
}
procedure TCAN232Form.FlagsButtonClick(Sender: TObject);
begin
    if ComPort.Connected then begin
        ComPort.WriteStr(#13+'F'+#13);  // Request Flags.
        Memo.Lines.Add('F');
    end;
end;


{
  Show About Dialog.
}
procedure TCAN232Form.About1Click(Sender: TObject);
begin
        About.Visible := TRUE;
end;

{
   Close Application and return to windows.
}
procedure TCAN232Form.Exit1Click(Sender: TObject);
begin
     Close;
end;

{
  If Close request arrives,  verify user wants to quit.
}
procedure TCAN232Form.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
     if (MessageDlg('Sure you want to quit?', mtConfirmation, [mbOk, mbCANCEL], 0) = mrOK) then begin
        CanClose := TRUE;
        ExtendedCloseLogFile;
     end
     else
        CanClose := FALSE;
end;

{
  Data logging.   -- Not impliemnted yet.
}
procedure TCAN232Form.NewLogFile1Click(Sender: TObject);
begin
    ExtendedNewLogFile(CAN232Device, Sender);
end;

{
  Data logging.   -- Not impliemnted yet.
}
procedure TCAN232Form.OpenLogFile1Click(Sender: TObject);
begin
    ExtendedOpenLogFile(CAN232Device, Sender);
end;

{
  Data logging.   -- Not impliemnted yet.
}
procedure TCAN232Form.SaveAsLogFileClick(Sender: TObject);
begin
    ExtendedSaveLogFile(CAN232Device, Sender);
end;

{
  Always save COM port parameters but CAN232 parameters are only saved in
  extended version of application.
}
procedure TCAN232Form.SaveParameters1Click(Sender: TObject);
begin
    ComPort.StoreSettings(stRegistry, 'HKEY_LOCAL_MACHINE\Software\CAN232');
    ExtendedSaveParam(CAN232Device, Sender);
end;

{
  Always load COM port parameters but CAN232 parameters are only loaded in
  extended version of application.
}
procedure TCAN232Form.LoadParameters1Click(Sender: TObject);
begin
    ComPort.LoadSettings(stRegistry, 'HKEY_LOCAL_MACHINE\Software\CAN232');
    ExtendedLoadParam(CAN232Device, Sender);
end;

{
  Called when application is first loaded.  If extended version exists then
  About Screen doesn't show on startup and MenuItems that function are enabled.
}
procedure TCAN232Form.FormActivate(Sender: TObject);
begin
   if ExtendedVersion then begin
        About.visible := FALSE;
   end;
end;

end.
